'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var compositionApi = require('@vue/composition-api');
var axios = _interopDefault(require('axios'));
var core = require('@vue-composable/core');

/* istanbul ignore next  */
const _axios = axios || (globalThis && globalThis.axios);
// TODO replace by shared project
const isObject = (d) => typeof d === 'object';
const isString = (d) => typeof d === 'string';
const isBoolean = (d) => typeof d === 'boolean';
function useAxios(configUrlThrowException, configThrowException, throwException = false) {
    /* istanbul ignore next  */
    
        !_axios &&
        console.warn(`[axios] not installed, please install it`);
    const config = !isString(configUrlThrowException) && !isBoolean(configUrlThrowException) ? configUrlThrowException : isObject(configThrowException) ? configThrowException : undefined;
    throwException = isBoolean(configUrlThrowException) ? configUrlThrowException : isBoolean(configThrowException) ? configThrowException : throwException;
    const axiosClient = _axios.create(config);
    const client = compositionApi.computed(() => axiosClient);
    const isCancelled = compositionApi.ref(false);
    const cancelledMessage = compositionApi.ref(null);
    let cancelToken = undefined;
    const cancel = (message) => {
        if (!cancelToken) {
            throw new Error('Cannot cancel because no request has been made');
        }
        cancelToken.cancel(message);
        isCancelled.value = true;
        if (message) {
            cancelledMessage.value = message;
        }
    };
    const use = core.usePromise(async (request) => {
        cancelToken = _axios.CancelToken.source();
        isCancelled.value = false;
        cancelledMessage.value = null;
        return axiosClient.request({
            cancelToken: cancelToken.token,
            ...request
        });
    }, throwException);
    const data = compositionApi.computed(() => (use.result.value && use.result.value.data) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.data) ||
        null);
    const status = compositionApi.computed(() => (use.result.value && use.result.value.status) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.status) ||
        null);
    const statusText = compositionApi.computed(() => (use.result.value && use.result.value.statusText) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.statusText) ||
        null);
    // if url provided in the config, execute it straight away
    // NOTE: `false` is passed to the `exec` to prevent the exception to be thrown
    if (typeof configUrlThrowException === 'string') {
        use.exec({ ...config, url: configUrlThrowException }, false);
    }
    else if (config && config.url) {
        use.exec(config, false);
    }
    return {
        ...use,
        client,
        data,
        status,
        statusText,
        cancel,
        isCancelled,
        cancelledMessage
    };
}

exports.useAxios = useAxios;
